@@ -18,12 +18,12 @@
  */
 
 #include <fcntl.h>
+#include <poll.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/ioctl.h>
-#include <sys/poll.h>
 #include <sys/soundcard.h>
 
 #include "oss.h"
