@@ -1,3 +1,22 @@
+/*
+ * Copyright (C) 2021 Mark Hills <mark@xwax.org>
+ *
+ * This file is part of "xwax".
+ *
+ * "xwax" is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License, version 3 as
+ * published by the Free Software Foundation.
+ *
+ * "xwax" is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ *
+ */
+
 /*
  * Illustrate clipping of characters
  */
@@ -48,7 +67,9 @@ int main(int argc, char *argv[])
     if (font == NULL)
         abort();
 
+#ifdef TTF_HINTING_NONE
     TTF_SetFontHinting(font, TTF_HINTING_NONE);
+#endif
     TTF_SetFontKerning(font, 1);
 
     surface = SDL_SetVideoMode(400, 200, 32, 0);
